using Adaptation.Shared;
using System;
using System.Data;
using System.Data.SqlClient;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading;

namespace Adaptation.FileHandlers.OpenInsight;

public class FromIQS
{

#nullable enable

    private static string GetCommandText(Logistics logistics, Stratus.Description description, string dateTime, long? subGroupId)
    {
        StringBuilder result = new();
        _ = result
                .AppendLine(" select ")
                .AppendLine("       se.f_sgrp se_sgrp, ")
                .AppendLine("       se.f_sgtm se_sgtm, ")
                .AppendLine("       se.f_tsno se_tsno, ")
                .AppendLine("       se.f_val se_val, ")
                .AppendLine("       rd.f_name rd_name, ")
                .AppendLine("       jd.f_name jd_name, ")
                .AppendLine("       pl.f_name pl_name, ")
                .AppendLine("       pd.f_name pd_name, ")
                .AppendLine("       td.f_test td_test, ")
                .AppendLine("       td.f_name td_name ")
                .AppendLine(" from [spcepiworld].[dbo].[sgrp_ext] se ")
                .AppendLine(" join [spcepiworld].[dbo].[prcs_dat] rd ")
                .AppendLine("   on se.f_prcs = rd.f_prcs ")
                .AppendLine(" join [spcepiworld].[dbo].[job_dat] jd ")
                .AppendLine("   on se.f_job = jd.f_job ")
                .AppendLine(" join [spcepiworld].[dbo].[part_lot] pl ")
                .AppendLine("   on se.f_lot = pl.f_lot ")
                .AppendLine(" join [spcepiworld].[dbo].[part_dat] pd ")
                .AppendLine("   on se.f_part = pd.f_part ")
                .AppendLine(" join [spcepiworld].[dbo].[test_dat] td  ")
                .AppendLine("   on se.f_test = td.f_test ")
                .AppendLine(" where se.f_flag = 0 ");
        if (subGroupId is not null)
            _ = result.Append("   and se.f_sgrp = ").Append(subGroupId).AppendLine("  ");
        _ = result
                .Append("   and rd.f_name = '").Append(description.Reactor).AppendLine("' ")
                .Append("   and pd.f_name = '").Append(description.PSN).AppendLine("' ")
                .AppendLine("   and jd.f_name in ('BIORAD4', 'BIORAD5') ")
                .Append("   and jd.f_name = '").Append(logistics.MesEntity).AppendLine("' ")
                .Append("   and pl.f_name = '").Append(description.RDS).AppendLine("' ")
                .Append("   and dateadd(HH, -7, (dateadd(SS, convert(bigint, se.f_sgtm), '19700101'))) = '").Append(dateTime).AppendLine("' ")
                .AppendLine("   for json path ");
        return result.ToString();
    }

    private static StringBuilder GetForJsonPath(string connectionString, string commandText)
    {
        StringBuilder stringBuilder = new();
        using SqlConnection sqlConnection = new(connectionString);
        sqlConnection.Open();
        using SqlCommand sqlCommand = new(commandText, sqlConnection);
        SqlDataReader sqlDataReader = sqlCommand.ExecuteReader(CommandBehavior.SequentialAccess);
        while (sqlDataReader.Read())
            _ = stringBuilder.Append(sqlDataReader.GetString(0));
        return stringBuilder;
    }

    internal static (long?, string) GetCommandText(string connectionString, Logistics logistics, Stratus.Description description, long breakAfter, long preWait)
    {
        string dateTime;
        string commandText;
        long? result = null;
        string dateFormat = Stratus.Description.GetDateFormat();
        if (DateTime.TryParseExact(description.Date, dateFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out DateTime dateTimeParsed))
            dateTime = dateTimeParsed.ToString("yyyy-MM-dd HH:mm:ss");
        else if (DateTime.TryParse(description.Date, CultureInfo.InvariantCulture, DateTimeStyles.None, out dateTimeParsed))
            dateTime = dateTimeParsed.ToString("yyyy-MM-dd HH:mm:ss");
        else
            dateTime = logistics.DateTimeFromSequence.ToString("yyyy-MM-dd HH:mm:ss");
        commandText = GetCommandText(logistics, description, dateTime, subGroupId: null);
        for (short i = 0; i < short.MaxValue; i++)
        {
            if (DateTime.Now.Ticks > preWait)
                break;
            Thread.Sleep(100);
        }
        StringBuilder stringBuilder = new();
        for (short z = 0; z < short.MaxValue; z++)
        {
            stringBuilder = GetForJsonPath(connectionString, commandText);
            if (stringBuilder.Length > 0)
                break;
            if (DateTime.Now.Ticks > breakAfter)
                // throw new Exception($"After {breakAfterSeconds} seconds, didn't find sub group id!");
                break;
            Thread.Sleep(250);
        }
        if (stringBuilder.Length == 0)
            commandText = stringBuilder.ToString();
        else
        {
            JsonElement[]? jsonElements = JsonSerializer.Deserialize<JsonElement[]>(stringBuilder.ToString());
            if (jsonElements is null || !jsonElements.Any() || jsonElements[0].ValueKind != JsonValueKind.Object)
                commandText = stringBuilder.ToString();
            else
            {
                JsonProperty[] jsonProperties = jsonElements[0].EnumerateObject().ToArray();
                if (!jsonProperties.Any() || jsonProperties[0].Name != "se_sgrp" || !long.TryParse(jsonProperties[0].Value.ToString(), out long subGroupId))
                    commandText = stringBuilder.ToString();
                else
                {
                    result = subGroupId;
                    commandText = GetCommandText(logistics, description, dateTime, subGroupId);
                }
            }
        }
        return new(result, commandText);
    }

#nullable disable

}