using Adaptation._Tests.Shared;
using Microsoft.Extensions.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Text;

namespace Adaptation._Tests.Static;

[TestClass]
public class Stratus : LoggingUnitTesting, IDisposable
{

#pragma warning disable CA2254
#pragma warning disable IDE0060

    internal static Stratus LoggingUnitTesting { get; private set; }

    public Stratus() : base(testContext: null, declaringType: null)
    {
        if (LoggingUnitTesting is null)
            throw new Exception();
    }

    public Stratus(TestContext testContext) : base(testContext, new StackFrame().GetMethod().DeclaringType)
    {
    }

    [ClassInitialize]
    public static void ClassInitialize(TestContext testContext) => LoggingUnitTesting ??= new Stratus(testContext);

    [ClassCleanup()]
    public static void ClassCleanup()
    {
        LoggingUnitTesting.Logger?.LogInformation("Cleanup");
        LoggingUnitTesting?.Dispose();
    }

    [TestMethod]
    public void TestDateTime()
    {
        DateTime dateTime = DateTime.Now;
        Assert.IsTrue(dateTime.ToString("M/d/yyyy h:mm:ss tt") == dateTime.ToString());
    }

    [TestMethod]
    public void TestDescriptor()
    {
        FileHandlers.Stratus.Descriptor descriptor;
        MethodBase methodBase = new StackFrame().GetMethod();
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Getting configuration"));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor(string.Empty);
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("12-123456-1234");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("1T123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("MP");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.Employee is "MP");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("12-123456-1234.2-1");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("12-123456-1234.02-1");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("20");
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("20.2");
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("20.2.1");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("20.1.1");
        Assert.IsTrue(descriptor.Layer is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("i171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.RDS is "i171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("o171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.RDS is "o171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("O171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.RDS is "O171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.Stratus.ProcessData.GetDescriptor("171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.RDS is "171308.1.51");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
    }

#if DEBUG
    [Ignore]
#endif
    [TestMethod]
    public void Staging()
    {
        MethodBase methodBase = new StackFrame().GetMethod();
        StringBuilder results = new();
        (string cellInstanceName, string cellInstanceVersionName)[] collection = new (string, string)[]
        {
            new("BIORAD4", "v2.47.0"),
            new("BIORAD5", "v2.47.0"),
        };
        string staging = "http://mestsa07ec.ec.local:9003/CellInstanceServiceV2";
        Shared.PasteSpecialXml.EAF.XML.API.CellInstance.CellInstanceVersion cellInstanceVersion;
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Getting configuration"));
        foreach ((string cellInstanceName, string cellInstanceVersionName) in collection)
        {
            cellInstanceVersion = AdaptationTesting.GetCellInstanceVersion($"{staging}/{cellInstanceName}/{cellInstanceVersionName}/configuration");
            _ = results.AppendLine($"{cellInstanceName}\t{cellInstanceVersionName}\t{cellInstanceVersion.EdaConnection.PortNumber}");
        }
        File.WriteAllText($"D:/Tmp/{methodBase.Module.Name}-{methodBase.ReflectedType.Name}-{methodBase.Name}.tsv", results.ToString());
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
    }

}